/*
 *  ElasticNet_Eff.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 4/5/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int ElasticNet_Eff(MissBin * MB, double lambda1, double lambda2, 
	gsl_vector * bhat, gsl_vector * sehat, int se_flag)
{
	size_t n = MB->G->size1 ;
	size_t d = MB->G->size2 ;
	MissBin * MB_aug = (MissBin *)malloc(sizeof(MissBin));
	
	gsl_vector * b0 = gsl_vector_calloc(d);
	gsl_vector * se0 = gsl_vector_calloc(d);
	gsl_vector * bstar = gsl_vector_calloc(d);
	gsl_vector * sestar = gsl_vector_calloc(d);
	gsl_vector * Yc ;
	gsl_matrix * Xc ; 
	gsl_vector * wtc ;
	size_t i,j;
	double scale1 = sqrt(1. + lambda2) ;
	double sqrt_lam2 = sqrt(lambda2);
	double ratio_lam2 = sqrt_lam2/scale1 ;
	double gamma = lambda1/scale1 ;
	int non0ind[d] ;
	int dnew = 0 ;
	
	// initialize MB_aug
	InitAllocMissBin(n+d,d,MB_aug);
	
	// setup data
	for (i=0;i<n;i++)
	{
		MB_aug->Y->data[i] = MB->Y->data[i] ;
		MB_aug->R->data[i] = MB->R->data[i] ;
		for (j=0;j<d;j++)
		{
			MB_aug->X->data[i * MB_aug->X->tda + j] = scale1 * MB->X->data[i * MB->X->tda + j] ;
			MB_aug->G->data[i * MB_aug->G->tda + j] = scale1 * MB->G->data[i * MB->G->tda + j] ;
		}
		for (j=0;j<2;j++)
		{
			MB_aug->lamhat->data[i * MB_aug->lamhat->tda + j] = MB->lamhat->data[i * MB->lamhat->tda + j] ;
		}
	}
	for (i=0;i<d;i++)
	{
		MB_aug->Y->data[n+i] = 0.0 ;
		MB_aug->R->data[n+i] = 99.0 ;
		MB_aug->X->data[(n+i) * MB_aug->X->tda + i] = ratio_lam2 ;
		MB_aug->G->data[(n+i) * MB_aug->G->tda + i] = ratio_lam2 ;
	}
	
	MB_aug->nr_eq1 = MB->nr_eq1 ;
	MB_aug->nr_eq2 = MB->nr_eq2 ;
	MB_aug->nr_eq99 = MB->nr_eq99 + d ;
	
	// estimate parameters for calculating L2
	//EstimatePsi(MB_aug);
	
	// get complete X,Y,weights
	Yc = gsl_vector_calloc(MB_aug->nr_eq99);
	Xc = gsl_matrix_calloc(MB_aug->nr_eq99,d);
	wtc = gsl_vector_calloc(MB_aug->nr_eq99);
	GetCompleteYX(Yc,Xc,wtc,MB_aug);
	
	/*
	printf("dim(lamhat) = %d-by-%d \n", (int) MB_aug->lamhat->size1, (int) MB_aug->lamhat->size2) ;
	for (i=n;i<(n+d);i++) 
	{
		printf("%d : lam1= %5.3f, lam2= %5.3f\n", (int) i, 
			gsl_matrix_get(MB_aug->lamhat,i,0), 
			gsl_matrix_get(MB_aug->lamhat,i,1));
	}
	for (i=(MB_aug->nr_eq99-d);i<MB_aug->nr_eq99;i++)
	{
		//printf("wt[%d] = %4.2f \n", (int) (n+i), wtc->data[n+i]) ;
		printf("%d : wt= %4.2f \n", (int) i, gsl_vector_get(wtc,i)) ;
	}
	*/
	
	// inital estimate b0
	EstimateBetaEFF(b0,se0,Yc,Xc,wtc,MB_aug);
	//printf("b0 :");
	//baj_print_vector(b0);
	
	// lasso on augmented data
	OLS_Eff_FITTER(b0,se0,Yc,Xc,wtc,MB_aug,gamma,2,25,bstar,sestar,se_flag);
	
	dnew=0;
	for (j=0;j<d;j++)
	{
		//double thj = 0.1 * se0->data[j] ;
		double thj = 0.01 ;
		double bj = scale1 * bstar->data[j] ;
		double bhj = bj ;
		
		//printf("b(%d) = %5.3f, th(%d) = %5.3f \n", (int) j, bj, (int) j, thj) ;
		if (baj_abs(bj) < thj) bhj = 0.0 ;
		else
		{
			non0ind[dnew] = j ;
			dnew += 1 ;
		}
		
		bhat->data[j] = bhj ;
		sehat->data[j] = scale1 * sestar->data[j] ;
	}

	
	
	gsl_vector_free(Yc);
	gsl_matrix_free(Xc);
	gsl_vector_free(wtc);
	gsl_vector_free(bstar);
	gsl_vector_free(sestar);
	gsl_vector_free(b0);
	gsl_vector_free(se0);
	free(MB_aug);
	
	return 0 ;
}
