/*
 *  ScoreHessianEFF.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/27/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int EstimateBetaEFF(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Yc, 
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB)
{
	//size_t n = MB->G->size1 ;
	size_t d = MB->G->size2 ;	
	gsl_vector * proj = gsl_vector_calloc(d);
	gsl_vector * score = gsl_vector_calloc(d);
	gsl_matrix * hess = gsl_matrix_calloc(d,d);
	gsl_vector * bhatI_local = gsl_vector_calloc(d);
	gsl_vector * sehatI = gsl_vector_calloc(d);
	gsl_vector * bold = gsl_vector_calloc(d);
	gsl_vector * bnew = gsl_vector_calloc(d);
	gsl_permutation * P = gsl_permutation_calloc(d);
	gsl_vector * newton_step = gsl_vector_calloc(d);
	int signum = 0 ;
	size_t iter = 0 ;
	int MAXITER = 3 ;
	double TOL = 1.E-8 ;
	double absdiff = 100.0 ;
	int j ;
	
	WLS_IPW(bhatI_local,sehatI,Yc,Xc,wtc);
	gsl_vector_memcpy(bnew,bhatI_local);
	gsl_vector_memcpy(MB->bhatI,bhatI_local);
	
	ScoreHessianIPW(score,hess,bnew,MB,proj);
	
	EstimateL2Projection(proj,bhatI_local,MB);
	gsl_vector_memcpy(MB->L2,proj);
	
	
	do
	{
		iter++;
		gsl_vector_memcpy(bold,bnew);
		
		/*
		printf("%d : ", (int) iter);
		baj_print_vector(bold);
		*/
		
		ScoreHessianIPW(score,hess,bold,MB,proj);
		
		gsl_linalg_LU_decomp(hess,P,&signum);
		gsl_linalg_LU_solve(hess,P,score,newton_step);
		
		for (j=0;j<d;j++) bnew->data[j] = bold->data[j] - newton_step->data[j] ;
		
		absdiff = MaxAbsDiff(bnew,bold);
		

		
	} while ((iter<MAXITER)&&(absdiff>TOL)) ;

	
	/*
	printf("Number of iterations = %d\n", (int) iter);
	printf("[Final] : ");
	baj_print_vector(bnew);
	*/
	
	gsl_vector_memcpy(bhat,bnew);
	EstimateVarBhatEff(sehat,bhat,Yc,Xc,wtc,MB);
	//baj_print_vector(sehat);
	
	gsl_vector_free(score);
	gsl_vector_free(sehatI);
	gsl_vector_free(bhatI_local);
	gsl_matrix_free(hess);
	gsl_vector_free(proj);
	gsl_permutation_free(P);
	gsl_vector_free(bnew);
	gsl_vector_free(bold);
	gsl_vector_free(newton_step);
	
	return 0 ;
}

