/*
 *  EstimateConditionalMoments.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/24/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

// m1 = E(X_1|Y, X_2, ..., X_d)
// m2 = E(X_1^2|Y, X_2, ..., X_d)
// Vobs = Y, X_2, ..., X_d
//
// conditional moment = mu1 + Sig12 * Sig22^{-1} * (Vobs - mu2)
//
int EstimateConditionalMomentsR2(double * cm1, gsl_vector * Vobs, 
	double mu1, gsl_vector * Sigma12, gsl_matrix * Sigma22, gsl_vector * mu2)
{
	size_t d = Sigma22->size2 ;
	gsl_vector * diff = gsl_vector_calloc(d);
	gsl_vector * Sigma22diff = gsl_vector_calloc(d);
	double prod = 0.0 ;
	double result = 0.0 ;
	int j ;	
	
	for (j=0;j<d;j++) diff->data[j] = Vobs->data[j] - mu2->data[j] ;
	
	printf("diff = ");
	baj_print_vector(diff);
	printf("\n");
	
	gsl_blas_dgemv(CblasNoTrans,1.0,Sigma22,diff,0.0,Sigma22diff);	
	printf("Sigma22diff = ");
	baj_print_vector(Sigma22diff);
	printf("\n");
	
	gsl_blas_ddot(Sigma12,Sigma22diff,&prod);
	
	printf("prod = %6.3f\n\n",prod);
	
	result = mu1 + prod ;
	*cm1 = result ;
	
	gsl_vector_free(diff);
	gsl_vector_free(Sigma22diff);
	
	return 0 ;
}

