/*
 *  EstimateL2Projection.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/24/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int EstimateL2Projection(gsl_vector * Proj, gsl_vector * beta, MissBin * MB)
{
	int i,j ;
	//int r ;
	size_t d = MB->X->size2 ;
	int totn = (int) MB->R->size ;
	double cm1_R2=0.0, cm2_R2=0.0 ;
	gsl_vector * Vobs_R1 = gsl_vector_calloc(d-1);
	gsl_vector * Vobs_R2 = gsl_vector_calloc(d);
	
	gsl_vector * proj_local = gsl_vector_calloc(d);
	gsl_vector * projI_R1 = gsl_vector_calloc(d);
	gsl_vector * projI_R2 = gsl_vector_calloc(d);
	
	gsl_vector * cm1_R1 = gsl_vector_calloc(2);
	gsl_matrix * cm2_R1 = gsl_matrix_calloc(2,2);
	gsl_matrix * alpha_R1 = gsl_matrix_calloc(d,2);
	gsl_vector * alpha_R2 = gsl_vector_calloc(d+1);
	gsl_matrix * tau_sq_R1 = gsl_matrix_calloc(2,2);
	double tau_sq_R2 = 0.0 ;
	

	PredictVmissLinearly_for_R1(alpha_R1,tau_sq_R1, MB);
	PredictVmissLinearly_for_R2(alpha_R2,&tau_sq_R2, MB);
	
	for (i=0;i<totn;i++)
	{
		double R = MB->R->data[i] ;
		double lam1 = MB->lamhat->data[i * MB->lamhat->tda + 0] ;
		double lam2 = MB->lamhat->data[i * MB->lamhat->tda + 1] ;
		double K1 = 1.-lam1 ;
		double wt1 = 0.0 ;
		
		// calculate weight
		if (baj_abs(R-1.0)< 0.1) wt1 = (1.0 - lam1)/K1 ;
		else wt1 = (0.0 - lam1)/K1 ;
		
		//printf("R[%d]=%d\n",i,(int) R);		
		
		for (j=0;j<d;j++)
		{
			projI_R1->data[j] = 0.0 ;
			projI_R2->data[j] = 0.0 ;
		}
		
			
			
		ExtractVobsForR1(Vobs_R1,i,MB->Y,MB->G);
		R1_PredictiveFunction(cm1_R1,cm2_R1,alpha_R1,tau_sq_R1,Vobs_R1);
		R1_IthContr_to_Proj(projI_R1,beta,cm1_R1,cm2_R1,Vobs_R1);
			
		gsl_vector_scale(projI_R1,wt1);
		gsl_vector_add(proj_local,projI_R1);
							
		if (R>1.5) 
		{
			double K2 = (1. - lam1) * (1. - lam2) ;
			double wt2 = 0.0 ;
			
			if (baj_abs(R-2.0)<0.1) wt2 = (1.0 - lam2)/K2 ;
			else wt2 = (0.0 - lam2)/K2 ;
			
			//printf("\t R[%d]=%d\n",i,(int) R);
			
			ExtractVobsForR2(Vobs_R2,i,MB->Y,MB->G);
			R2_PredictiveFunction(&cm1_R2,&cm2_R2,alpha_R2,tau_sq_R2,Vobs_R2);
			R2_IthContr_to_Proj(projI_R2,beta,cm1_R2,cm2_R2,Vobs_R2);
			
			gsl_vector_scale(projI_R2,wt2);
			gsl_vector_add(proj_local,projI_R2);
		}
		
	}
	
	gsl_vector_scale(proj_local,-1.0);
	
	gsl_vector_memcpy(Proj,proj_local);
	
	gsl_vector_free(Vobs_R1);
	gsl_vector_free(Vobs_R2);
	gsl_vector_free(proj_local);
	gsl_vector_free(projI_R1);
	gsl_vector_free(projI_R2);
	gsl_vector_free(alpha_R2);
	gsl_matrix_free(alpha_R1);
	gsl_matrix_free(tau_sq_R1);
	gsl_vector_free(cm1_R1);
	gsl_matrix_free(cm2_R1);
	
	return 0 ;
}

