/*
 *  EstimatePsi.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/19/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int EstimatePsi1(MissBin * MB);
int EstimatePsi2(MissBin * MB);

int EstimatePsi(MissBin * MB)
{
	EstimatePsi1(MB);
	EstimatePsi2(MB);
	return 0 ;
}

int EstimatePsi1(MissBin * MB)
{
	size_t nr = MB->G->size1 ;
	size_t dr = MB->G->size2 ;
	int i,j ;
	gsl_vector * tmpY = gsl_vector_calloc(nr);
	gsl_matrix * tmpZ = gsl_matrix_calloc(nr,dr);
	gsl_vector * psi = gsl_vector_calloc(dr);
	gsl_vector * psi_se = gsl_vector_calloc(dr);
	gsl_vector * mu = gsl_vector_calloc(nr);
	
	for (i=0;i<nr;i++)
	{
		if (MB->R->data[i]<1.5) tmpY->data[i] = 1.0 ;
		else tmpY->data[i] = 0.0 ;
				
		tmpZ->data[i*tmpZ->tda + 0] = 1.0 ;
		tmpZ->data[i*tmpZ->tda + 1] = MB->Y->data[i] ;
		for (j=0;j<(dr-2);j++) tmpZ->data[i*tmpZ->tda + 2 + j] = gsl_matrix_get(MB->G,i,2+j) ;
	}
	
	LogisticRegression(psi,psi_se,tmpY,tmpZ);
	for (j=0;j<dr;j++) MB->psi1->data[j] = psi->data[j] ;
		
	//baj_print_vector(psi);
	//baj_print_vector(psi_se);
		
	if (1)
	{
		gsl_blas_dgemv(CblasNoTrans,1.0,tmpZ,psi,0.0,mu);
		for (i=0;i<nr;i++)
		{
			double tmp = 1./(1. + exp(-1.0 * mu->data[i])) ;
			gsl_matrix_set(MB->lamhat,i,0,tmp) ;
			//MB->lamhat->data[i*MB->lamhat->tda + 0] = tmp ;
		}
	}
	
	gsl_vector_free(mu);
	gsl_vector_free(tmpY);
	gsl_matrix_free(tmpZ);
	gsl_vector_free(psi);
	gsl_vector_free(psi_se);
	return 0 ;
}

int EstimatePsi2(MissBin * MB)
{
	size_t totn = MB->G->size1 ;
	size_t nr = totn - MB->nr_eq1 ;
	size_t dr = 1 + MB->G->size2 ;
	int i,j ;
	gsl_vector * tmpY = gsl_vector_calloc(nr);
	gsl_matrix * tmpZ = gsl_matrix_calloc(nr,dr);
	gsl_vector * psi = gsl_vector_calloc(dr);
	gsl_vector * psi_se = gsl_vector_calloc(dr);
	gsl_vector * mu = gsl_vector_calloc(nr);
	int ppt_cntr = 0 ;
	
			
	for (i=0;i<totn;i++)
	{
		double Ri = MB->R->data[i] ;
		if (Ri>1.5)
		{
			if (Ri<2.5) tmpY->data[ppt_cntr] = 1.0 ;
			else tmpY->data[ppt_cntr] = 0.0 ;

			tmpZ->data[ppt_cntr*tmpZ->tda + 0] = 1.0 ;
			tmpZ->data[ppt_cntr*tmpZ->tda + 1] = MB->Y->data[i] ;
			for (j=0;j<(dr-2);j++) tmpZ->data[ppt_cntr*tmpZ->tda + 2 + j] = gsl_matrix_get(MB->G,i,1+j) ;
			
			ppt_cntr++ ;
		}
	}
		
		
	LogisticRegression(psi,psi_se,tmpY,tmpZ);
	for (j=0;j<dr;j++) MB->psi2->data[j] = psi->data[j] ;
		
	//baj_print_vector(psi);
	//baj_print_vector(psi_se);
		
		
	if (1)
	{
		gsl_blas_dgemv(CblasNoTrans,1.0,tmpZ,psi,0.0,mu);
		ppt_cntr = 0 ;
		for (i=0;i<totn;i++)
		{
			double Ri = MB->R->data[i] ;
			double tmp = 0.0 ;
			if (Ri>1.5) 
			{
				tmp = 1./(1. + exp(-1.0 * mu->data[ppt_cntr])) ;
				ppt_cntr++;
			}
			//MB->lamhat->data[i*MB->lamhat->tda + 1] = tmp ;
			gsl_matrix_set(MB->lamhat,i,1,tmp) ;
		}
	}

	gsl_vector_free(tmpY);
	gsl_matrix_free(tmpZ);
	gsl_vector_free(psi);
	gsl_vector_free(psi_se);
	gsl_vector_free(mu);
	
	return 0 ;
}
