/*
 *  EstimateTheta.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/19/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int EstimateTheta(MissBin * MB)
{
	int i,j,k ;
	size_t n = MB->G->size1 ;
	size_t d = MB->G->size2 ;
	gsl_vector * local_mu = gsl_vector_calloc(d+1);
	gsl_matrix * local_sigma = gsl_matrix_calloc(d+1,d+1);
	gsl_vector * sumX = gsl_vector_calloc(d+1);
	gsl_matrix * sumX2 = gsl_matrix_calloc(d+1,d+1);
	double denom = 0.0 ;
	
	for (i=0;i<n;i++)
	{
		if (MB->R->data[i] > 2.5)
		{
			double lam1 = MB->lamhat->data[i*MB->lamhat->tda + 0] ;
			double lam2 = MB->lamhat->data[i*MB->lamhat->tda + 1] ;
			double margin = (1.0 - lam1) * (1.0 - lam2) ;
			
			/*
			if (i<10) 
			{
				printf("lam1 = %6.3e, lam2 = %6.3e, marginal prob[%d] = %6.3e\n", lam1,lam2,i, margin);
			}
			*/
			for (j=0;j<(d+1);j++)
			{
				double tmp_xij = 0.0 ;
				
				if (j==0) tmp_xij = MB->Y->data[i] ;
				else tmp_xij = MB->X->data[i * MB->X->tda + j - 1];
				
				sumX->data[j] += tmp_xij/margin ;
				
				for (k=0;k<(d+1);k++)
				{
					double tmp_xik = 0.0 ;
					
					if (k==0) tmp_xik = MB->Y->data[i] ;
					else tmp_xik = MB->X->data[i * MB->X->tda + k - 1];
					
					sumX2->data[j * sumX2->tda + k] += tmp_xij * tmp_xik / margin ;
				}
			}
			denom += 1.0 / margin ;
		}
	}
	
	//printf("denom = %9.6f\n", denom);
	
	for (j=0;j<(d+1);j++) 
	{
		double tmp_xjbar = sumX->data[j]/denom ;
		local_mu->data[j] = tmp_xjbar ;
		
		for (k=0;k<(d+1);k++) 
		{
			double tmp_xkbar = sumX->data[k]/denom ;
			double tmp_xjk = sumX2->data[j * sumX2->tda + k]/denom ;
			local_sigma->data[j*local_sigma->tda + k] = tmp_xjk - tmp_xjbar * tmp_xkbar ;
		}
		
	}
	
	//baj_print_vector(local_mu);
	//printf("\n\n");
	//baj_print_matrix(local_sigma);
	
	gsl_vector_memcpy(MB->theta_mu, local_mu);
	gsl_matrix_memcpy(MB->theta_sigma, local_sigma);
	
	gsl_vector_free(sumX);
	gsl_matrix_free(sumX2);
	gsl_vector_free(local_mu);
	gsl_matrix_free(local_sigma);
			
	return 0 ;
}

