/*
 *  FindPenStdErrorsOLSEff.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/30/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int EstimateVhatEffGradient_Reduced(gsl_matrix * I0, int * non0ind_new, int d_new,
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB);

int EstimateVhatEffScoreVar_Reduced(gsl_matrix * I1, gsl_vector * beta, 
	int * non0ind_new, int d_new, MissBin * MB);

int FindPenStdErrorsOLSEff(gsl_vector * se, gsl_vector * b_all, int * non0ind_new, int d_new, 
	double lambda, int METH, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, 
	gsl_vector * b0)
{
    size_t n = MB->G->size1;
    gsl_vector * bred = gsl_vector_calloc(d_new);
    gsl_matrix * nSigmaU = gsl_matrix_calloc(d_new, d_new) ;
    gsl_matrix * I0 = gsl_matrix_calloc(d_new, d_new);
	gsl_matrix * I1 = gsl_matrix_calloc(d_new, d_new);
    gsl_matrix * V = gsl_matrix_calloc(d_new, d_new);
    int j;
    
    //printf("FindPenStdErrorsOLSEff()\n");
	
    for (j=0;j<d_new;j++) 
	{
		bred->data[j] = b_all->data[*(non0ind_new+j)] ; // Reduced bhat
	}

    EvalnSigmaU2(nSigmaU, lambda, 3.7, bred, METH, (double) n, b0) ; 
    
    //printf("EstimateVhatEffGradient_Reduced()\n");
    EstimateVhatEffGradient_Reduced(I0,non0ind_new,d_new,Xc,wtc,MB);
    //printf("EstimateVhatEffScoreVar_Reduced()\n");
	EstimateVhatEffScoreVar_Reduced(I1,b_all,non0ind_new,d_new,MB);
    //printf("Finished I0,I1\n");
	
    gsl_matrix_add(I0,nSigmaU) ; // (I0 + nSigmaU)
	
	SandwichEstimate(V,I0,I1);
    
    gsl_vector_set_zero(se);
    for (j=0;j<d_new;j++)
    {
        int indj = *(non0ind_new+j) ;
        double sisj = V->data[j * V->tda + j] ;
        se->data[indj] = sqrt(sisj) ;
    }

    gsl_matrix_free(nSigmaU);
    gsl_matrix_free(I0);
	gsl_matrix_free(I1);
    gsl_matrix_free(V);
    gsl_vector_free(bred);
    
    return 0 ;
}

int EstimateVhatEffGradient_Reduced(gsl_matrix * I0, int * non0ind_new, int d_new,
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB)
{
	int i ;
	size_t nc = Xc->size1 ;
	gsl_matrix * W = gsl_matrix_calloc(nc,nc);
	gsl_matrix * RedX = gsl_matrix_calloc(nc,d_new);
	gsl_matrix * WX = gsl_matrix_calloc(nc,d_new);
	gsl_matrix * XtWX = gsl_matrix_calloc(d_new,d_new);
	
	for (i=0;i<nc;i++) W->data[i*W->tda + i] = wtc->data[i] ;
	
    GetReducedX(RedX, non0ind_new, d_new, Xc) ; 

	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,W,RedX,0.0,WX);
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,RedX,WX,0.0,XtWX);
	
	gsl_matrix_memcpy(I0,XtWX);
	
	gsl_matrix_free(W);
	gsl_matrix_free(WX);
	gsl_matrix_free(XtWX);
	gsl_matrix_free(RedX);
	return 0;
}

int EstimateVhatEffScoreVar_Reduced(gsl_matrix * I1, gsl_vector * beta, 
	int * non0ind_new, int d_new, MissBin * MB)
{
	size_t n = MB->G->size1 ;
	size_t totd = MB->G->size2 ;
	gsl_matrix * D = gsl_matrix_calloc(n,totd);
	gsl_matrix * Dred = gsl_matrix_calloc(n,d_new);
	gsl_matrix * I1_local = gsl_matrix_calloc(d_new,d_new);
	int i,j,k ;
	
	GetScoreEffFullMatrix(D,beta,MB);
	GetReducedX(Dred,non0ind_new,d_new,D);
	
	for (i=0;i<n;i++)
	{
		for (j=0;j<d_new;j++) 
		{
			double dj = D->data[i*D->tda + j] ;
			for (k=0;k<d_new;k++) 
			{
				double dk = D->data[i*D->tda + k] ;
				I1_local->data[j * I1_local->tda + k] += dj * dk ;
			}
		}
	}
	
	gsl_matrix_memcpy(I1,I1_local);
	
	gsl_matrix_free(I1_local);
	gsl_matrix_free(D);
	gsl_matrix_free(Dred);
	return 0 ;
}
