/*
 *  GCV_OLS_Eff.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/30/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

double GCV_OLS_Eff_ObjFun(gsl_vector * b0, gsl_vector * se0, gsl_vector * Yc, gsl_matrix * Xc, 
	gsl_vector * wtc, MissBin * MB, double lambda, int METH, int NSTEP)
{
    size_t n = MB->Y->size ;
    size_t d0 = b0->size ;
    int i,k ;
    double gcv_num = 100.0, gcv_den = 100., curr_gcv = 100. ;
	double sse=0.0;
	int dsig = 0;
	double trp = 0.0;
    gsl_vector * bret = gsl_vector_calloc(d0);
    gsl_vector * seret = gsl_vector_calloc(d0);
    gsl_vector * yhat = gsl_vector_calloc(n);
    
	
	OLS_Eff_FITTER(b0, se0, Yc, Xc, wtc, MB, lambda, METH, NSTEP, bret, seret, 0) ;
		
	for (k=0;k<bret->size;k++) 
	{
		if (baj_abs(*(bret->data + k)) > 1.E-6) dsig++ ;
	}
		
	sse = ResidualSSWeighted(Yc,Xc,wtc,bret,MB); 
        
	if (dsig<1) trp=0.0;
	else trp = GetTrProjIPW(bret, Xc, wtc, MB, lambda, 0.0, METH, b0);
        
	//gcv_num = sse/((double) n);
	gcv_num = sse;
	gcv_den = pow(1.0-(trp/((double) n)),2.0);
	curr_gcv = gcv_num/gcv_den ;
	/*
	printf("%d GCV(%5.3f) = %5.3f, trp = %5.3f, sse = %5.3f \n", METH, lambda, curr_gcv, 
		trp, sse) ;
    */
	
    gsl_vector_free(bret);
    gsl_vector_free(seret);
    gsl_vector_free(yhat);
    
    return curr_gcv ;
}


double GCV_OLS_Eff(gsl_vector * b0, gsl_vector * se0, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, int METH, 
	int NSTEP)
{
    double lam_gcv = 0.0 ;
    double min_gcv = 1.0E6 ;
	int gcvlen = 7 ;
    double lambda[gcvlen];
    double gcv[gcvlen];
    int j ;
    
	GetLamRange(lambda, gcvlen, b0, MB->Y, MB->X, METH);
	
    for (j=0;j<gcvlen;j++)
    {
		double curr_gcv = GCV_OLS_Eff_ObjFun(b0,se0,Yc,Xc,wtc,MB,*(lambda+j),METH,NSTEP) ;
		*(gcv+j) = curr_gcv ;
		//printf("GCV(%6.3f) = %6.3e\n", *(lambda+j), curr_gcv);
		if (*(gcv+j) < min_gcv)
		{
			min_gcv = *(gcv+j) ;
			lam_gcv = *(lambda+j) ;
		}
    } 
    
    return lam_gcv ;
}

double GCV_OLS_Eff_Wages(gsl_vector * b0, gsl_vector * se0, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, int METH, 
	int NSTEP, double L_lo, double L_hi)
{
    double lam_gcv = 0.0 ;
    double min_gcv = 1.0E6 ;
	int gcvlen = 13 ;
    double lambda[gcvlen];
    double gcv[gcvlen];
	double stepsize = (L_hi - L_lo)/((double) gcvlen) ; 
    int j ;
    
	//GetLamRange(lambda, gcvlen, b0, MB->Y, MB->X, METH);
	
	for (j=0;j<gcvlen;j++) lambda[j] = L_lo + stepsize * j ;
    for (j=0;j<gcvlen;j++)
    {
		double curr_gcv = GCV_OLS_Eff_ObjFun(b0,se0,Yc,Xc,wtc,MB,*(lambda+j),METH,NSTEP) ;
		*(gcv+j) = curr_gcv ;
		printf("GCV(%6.3f) = %6.3e\n", *(lambda+j), curr_gcv);
		if (*(gcv+j) < min_gcv)
		{
			min_gcv = *(gcv+j) ;
			lam_gcv = *(lambda+j) ;
		}
    } 
    
    return lam_gcv ;
}
