/*
 *  GCV_OLS_Full.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/22/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

double GCV_OLS_Full_ObjFun(gsl_vector * b0, gsl_vector * se0, gsl_vector * Y, gsl_matrix * X, 
	double lambda, int METH, int NSTEP)
{
    size_t n = Y->size ;
    size_t d0 = b0->size ;
    int i,k ;
    double gcv_num = 100.0, gcv_den = 100., curr_gcv = 100. ;
	double sse=0.0;
	//double diff=0.0;
	int dsig = 0;
	double trp = 0.0;
	//int diff_flag = 0;
    gsl_vector * bret = gsl_vector_calloc(d0);
    gsl_vector * seret = gsl_vector_calloc(d0);
    gsl_vector * yhat = gsl_vector_calloc(n);
    
	
	OLS_Full_FITTER(b0, se0, Y, X, lambda, METH, NSTEP, bret, seret, 0) ;
		
	for (k=0;k<bret->size;k++) 
	{
		if (baj_abs(*(bret->data + k)) > 1.E-6) dsig++ ;
	}
		
	gsl_blas_dgemv(CblasNoTrans, 1.0, X, bret, 0.0, yhat);

	sse = 0.0 ;
	for (i=0;i<n;i++) sse += pow((Y->data[i] - yhat->data[i]), 2.0);
        
	if (dsig<1) trp=0.0;
	else trp = GetTrProj(bret, X, lambda, METH);
        
	gcv_num = sse/((double) n);
	gcv_den = pow(1.0-(trp/((double) n)),2.0);
	curr_gcv = gcv_num/gcv_den ;
	
    
    gsl_vector_free(bret);
    gsl_vector_free(seret);
    gsl_vector_free(yhat);
    
    return curr_gcv ;
}


double GCV_OLS_Full(gsl_vector * b0, gsl_vector * se0, gsl_vector * Y, gsl_matrix * X, int METH, 
	int NSTEP)
{
    double lam_gcv = 0.0 ;
    double min_gcv = 1.0E6 ;
	int gcvlen = 20 ;
    double lambda[gcvlen];
    double gcv[gcvlen];
    int j ;
    
	GetLamRange(lambda, gcvlen, b0, Y, X, METH);
	
    for (j=0;j<gcvlen;j++)
    {
		*(gcv+j) = GCV_OLS_Full_ObjFun(b0,se0,Y,X,*(lambda+j),METH,NSTEP) ;
		//printf("GCV(%6.3f) = %6.3f\n", *(lambda+j), *(gcv+j));
		if (*(gcv+j) < min_gcv)
		{
			min_gcv = *(gcv+j) ;
			lam_gcv = *(lambda+j) ;
		}
    } 
    
    return lam_gcv ;
}
