/*
 *  GetLamRange.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/23/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int GetLamRange(double * lambda, int len_lambda, gsl_vector * b0, gsl_vector * Y, gsl_matrix * X, int METH)
{
	size_t d0 = b0->size;
	size_t n = Y->size;
    double rootn = sqrt(n);
    double Llo=0.1, Lhi=10.0 ;
	double sse=0.0, mse=0.0, stepsize=0.0, se=0.0;
	int i,j ;
    gsl_vector * yhat = gsl_vector_calloc(n);
	
    gsl_blas_dgemv(CblasNoTrans, 1.0, X, b0, 0.0, yhat);
	
	sse=0.0 ;
    for (i=0;i<n;i++) sse += pow((Y->data[i] - yhat->data[i]), 2.0);
    mse=sse/((double) (n-d0));

    switch (METH)
    {
        
        case 0: // scad
        {
            Llo = 0.4 ; // orig
            Lhi = 2.0 ; // orig
            break ;
        }
        case 1: // hard
        {
            Llo = 1.2;
            Lhi = 3.0;
            break ;
        }
        case 2: // lasso
        {
            Llo = 0.2; // orig
            Lhi = 1.8; // orig
            break ;
        }
		case 3 : // alasso
		{
			Llo = 0.4 ;
			Lhi = 2.0 ;
			break ;
		}
		case 4 : // elastic net
		{
			Llo = 0.01 ;
			Lhi = 5.0 ;
			break ;
		}
		
        default :
        printf("Unknown call to METH in GetLamRange\n") ;
        exit(8) ;
    }
    
    
    stepsize = (Lhi - Llo)/((double) len_lambda) ;
    se = sqrt(mse)/rootn ;
    for (j=0;j<len_lambda;j++) *(lambda + j) = (Llo + j * stepsize) * se ;

	gsl_vector_free(yhat);
	
	return 0 ;
}
