/*
 *  GetTrProjIPW.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/30/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

double GetTrProjIPW(gsl_vector * b, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, double lambda1, 
	double lambda2, int METH, gsl_vector * b0)
{
    int j ;
	size_t totn = MB->Y->size ;
    size_t d0 = (int) b->size ;
    size_t nc = Xc->size1 ;
    double bg[d0];
	double b0g[d0];
    int non0g[d0];
    int g_cntr = 0 ;
	double trproj = 0.0 ;
    int i ;
	
    for (j=0;j<d0;j++)
    {
        double tj = b->data[j];
        if (baj_abs(tj)>1.E-3) 
        {
            *(bg+g_cntr) = tj ;
			*(b0g+g_cntr) = b0->data[j] ;
            *(non0g+g_cntr) = j ;
            g_cntr++;
        }
    }
	
	if (g_cntr < 1) trproj = 0.0 ;
	else
	{
		// Reduced X
		// ---------
		gsl_matrix * RedX = gsl_matrix_calloc(nc,g_cntr) ;
		gsl_matrix * W = gsl_matrix_calloc(nc,nc);
		gsl_matrix * WX = gsl_matrix_calloc(nc,g_cntr);
		gsl_matrix * XtWX = gsl_matrix_calloc(g_cntr, g_cntr) ;
		gsl_matrix * Proj = gsl_matrix_calloc(g_cntr,g_cntr);
		gsl_matrix * nSigmaU = gsl_matrix_calloc(g_cntr, g_cntr) ;
    
		for (i=0;i<nc;i++) W->data[i*W->tda + i] = wtc->data[i] ;
	
		GetReducedX(RedX, non0g, g_cntr, Xc) ;
    
		gsl_vector_view bbef = gsl_vector_view_array(bg, g_cntr) ;	
		gsl_vector_view b0b = gsl_vector_view_array(b0g, g_cntr) ;
	
		if (METH<4) EvalnSigmaU2(nSigmaU, lambda1, 3.7, &bbef.vector, METH, (double) totn, &b0b.vector) ;
		else
			{
			for (j=0;j<g_cntr;j++)
			{
				double x = *(bg+j);
				double absx = baj_abs(x) ;
				double tmp = totn*lambda1/(1.E-5+absx) + totn*lambda2 ;
				nSigmaU->data[j*nSigmaU->tda + j] = tmp ;
			}
		}
	
		gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,W,RedX,0.0,WX);
		gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,RedX,WX,0.0,XtWX);
		gsl_matrix_add(nSigmaU,XtWX);
		AunderB(nSigmaU,XtWX,Proj);
	
		trproj = 0.0 ;	
		for (j=0;j<g_cntr;j++) trproj += Proj->data[j*Proj->tda + j];

		gsl_matrix_free(nSigmaU) ;
		gsl_matrix_free(RedX) ;
		gsl_matrix_free(XtWX);
		gsl_matrix_free(Proj);
		gsl_matrix_free(W);
		gsl_matrix_free(WX);
    }
	
	
    return trproj ;
}
