/*
 *  LogisticRegression.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/19/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int LogisticRegressionScoreHessian(gsl_vector * score, gsl_matrix * hess, gsl_vector * beta,
	gsl_vector * Y, gsl_matrix * X);
int LogisticRegression(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Y, gsl_matrix * X)
{
	size_t n = X->size1 ;
	size_t d = X->size2 ;
	gsl_vector * bnew = gsl_vector_calloc(d);
	gsl_vector * bold = gsl_vector_calloc(d);
	gsl_vector * score = gsl_vector_calloc(d);
	gsl_matrix * hess = gsl_matrix_calloc(d,d);
	
	size_t iter = 0 ;
	int MAXITER = 50 ;
	double TOL = 1.E-8 ;
	double alpha_init = 0.0, sum_Y = 0.0, absdiff = 100.0 ;
	int i ;
	
	gsl_permutation * P = gsl_permutation_calloc(d);
	int signum ;
		
	// initial values
	for (i=0;i<n;i++) sum_Y += Y->data[i] ;
	alpha_init = log(sum_Y/((double) n - sum_Y));
	bnew->data[0] = alpha_init ;
	
	do
	{
		iter++;
		gsl_vector_memcpy(bold,bnew);
		//baj_print_vector(bold);
		
		LogisticRegressionScoreHessian(score,hess,bold,Y,X);
		
		gsl_linalg_LU_decomp(hess,P,&signum);
		gsl_linalg_LU_solve(hess,P,score,bnew);
		//gsl_vector_scale(bnew,-1.0);
		gsl_vector_add(bnew,bold);
		
		absdiff = MaxAbsDiff(bnew,bold);
		
	} while ((iter<MAXITER)&&(absdiff>TOL)) ;
	
	for (i=0;i<d;i++) bhat->data[i] = bnew->data[i] ;

	if(1)
	{
		gsl_matrix * Diag = gsl_matrix_calloc(d,d);
		gsl_matrix * V = gsl_matrix_calloc(d,d);
		for (i=0;i<d;i++) gsl_matrix_set(Diag,i,i,1.0);
	
		LogisticRegressionScoreHessian(score,hess,bnew,Y,X);
		AunderB(hess,Diag,V);
		
		for (i=0;i<d;i++) sehat->data[i] = sqrt(V->data[i*V->tda + i]) ;
	
		gsl_matrix_free(Diag);
		gsl_matrix_free(V);
	}
	

	gsl_vector_free(bold);
	gsl_vector_free(bnew);
	gsl_vector_free(score);
	gsl_matrix_free(hess);
	gsl_permutation_free(P);
	
	return 0 ;
}

int LogisticRegressionScoreHessian(gsl_vector * score, gsl_matrix * hess, gsl_vector * beta,
	gsl_vector * Y, gsl_matrix * X)
{
	size_t n = X->size1 ;
	size_t d = X->size2 ;
	gsl_vector * lp = gsl_vector_calloc(n);
	int k,j,i ;
	
	gsl_blas_dgemv(CblasNoTrans,1.0,X,beta,0.0,lp);
	
	// clear memory
	for (i=0;i<d;i++)
	{
		score->data[i] = 0.0 ;
		for (j=0;j<d;j++) hess->data[i*hess->tda + j] = 0.0 ;
	}
	
	// calculate score,hessian
	for (i=0;i<n;i++)
	{
		double pii = 1.0/(1.0 + exp(-1.0 * lp->data[i])) ;
		double eii = Y->data[i] - pii ;
		for (j=0;j<d;j++)
		{
			double xij = X->data[i*X->tda + j] ;
			score->data[j] += xij * eii ;
			for (k=0;k<d;k++) hess->data[j*hess->tda + k] += xij * X->data[i*X->tda + k] * pii * (1.0-pii) ;
		}
	}
	
	// divide by n? shouldn't matter
	
	gsl_vector_free(lp);
	return 0 ;
}
