/*
 *  OLS_IPW_FITTER.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/23/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int Shrink_OLS_IPW_Bhat(double * beta_new, gsl_vector * Yc, gsl_matrix * Xc, 
	gsl_vector * wtc, MissBin * MB, 
	double * beta_old, int d_old, int * non0ind_old, double lambda, int METH);
int Find_OLS_IPW_Thresholds(double * th_new, int d_new, int * non0ind_new, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB) ;
int OLS_IPW_FITTER(gsl_vector * b0, gsl_vector * se0, gsl_vector * Yc, 
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, 
    double lambda, int METH, int NSTEP, gsl_vector * bret, gsl_vector * seret, int se_flag)
{
    int j ;
    int d = (int) b0->size ;
    int d_new = (int) b0->size ;
    int d_old=0, d_cntr=0 ;
    size_t step=0 ;
    //size_t n = Y->size ;
    double TOL = 1.e-4 ;
    double th_new[d] ;
    double th_old[d] ;
    double beta_new[d] ;
    double beta_old[d] ;
    gsl_vector * bnew = gsl_vector_calloc(d);
    gsl_vector * bold = gsl_vector_calloc(d);
    int non0ind_new[d] ;
    int non0ind_old[d] ;
    double norm_diff = 10. ;
	
	d_cntr=0;
	for (j=0;j<d;j++)
    {
        double betaj = b0->data[j] ;
        double th0 = 0.5 * se0->data[j];
        if (baj_abs(betaj)>th0)
        {
            bnew->data[j] = betaj;
            *(non0ind_new+d_cntr) = j ;
            *(beta_new+d_cntr) = betaj ;
            *(th_new+d_cntr) = th0 ;
            d_cntr++;
        }
    }
    d_new = d_cntr;

    do
    {
        step++ ;
		
        gsl_vector_set_zero(bold);
        for (j=0;j<d_new;j++)
        {
            *(beta_old+j) = *(beta_new+j) ;
            *(th_old+j) = *(th_new+j) ;
            *(non0ind_old+j) = *(non0ind_new+j) ;
            bold->data[*(non0ind_new+j)] = *(beta_new+j);
        }
        d_old = d_new ;
        
		/*
        printf("%d : ", (int) step) ;
        for (j=0;j<d_old;j++) printf("%6.4f ", *(beta_old+j)) ;
        printf("\n") ;
        */
		       
        if (d_old < 1)
        {
            fprintf(stderr,"No variables left in model .... break\n") ;
			d_new = 0;
			gsl_vector_set_zero(bnew);
			break ;
        }
        else 
		{
			Shrink_OLS_IPW_Bhat(beta_new, Yc, Xc, wtc, MB, beta_old, d_old, non0ind_old, lambda, METH);
		}
		
        d_cntr=0;
        gsl_vector_set_zero(bnew);
        for (j=0;j<d_old;j++) 
        {
            double betaj = *(beta_new+j) ;
            if (baj_abs(betaj)>*(th_old+j))
            { 
                bnew->data[*(non0ind_old+j)] = betaj ;
                *(non0ind_new+d_cntr) = *(non0ind_old+j) ;
                *(beta_new+d_cntr) = betaj ;
                d_cntr++;
            }
        }
        d_new = d_cntr;
        
        if (d_new>0) Find_OLS_IPW_Thresholds(th_new, d_new, non0ind_new, Yc, Xc, wtc, MB) ;
        norm_diff = MaxAbsDiff(bold,bnew) ; 

    } while ((norm_diff>TOL) && (step<NSTEP)) ;
        
    gsl_vector_memcpy(bret, bnew) ;
	gsl_vector_set_zero(seret);
	
	if ((d_new>0)&&(se_flag==1))
	{
		//FindPenStdErrorsOLSIPW(seret, bnew, non0ind_new, d_new, Yc, Xc, lambda, METH) ;
    }
	
    gsl_vector_free(bold);
    gsl_vector_free(bnew);
    
    return 0 ;
}

int Find_OLS_IPW_Thresholds(double * th_new, int d_new, int * non0ind_new, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB) 
{
	int j ;
	size_t nc = Xc->size1 ;
	gsl_vector * bols = gsl_vector_calloc(d_new);
	gsl_vector * seols = gsl_vector_calloc(d_new);
	gsl_matrix * RedX = gsl_matrix_calloc(nc, d_new);
            
	GetReducedX(RedX, non0ind_new, d_new, Xc) ; // new reduced X
            
	WLS_IPW(bols,seols,Yc,Xc,wtc);
	            
	for (j=0;j<d_new;j++) *(th_new + j) = 0.5 * seols->data[j];
                
	gsl_matrix_free(RedX);
	gsl_vector_free(bols);
	gsl_vector_free(seols);

	return 0 ;
}

int Shrink_OLS_IPW_Bhat(double * beta_new, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, 
	MissBin * MB, 
	double * beta_old, int d_old, int * non0ind_old, double lambda, int METH)
{
	size_t nc = Yc->size ;
	int signum = 0;
	gsl_matrix * W = gsl_matrix_calloc(nc,nc);
	gsl_matrix * XtW = gsl_matrix_calloc(d_old,nc);
	gsl_matrix * XtWX = gsl_matrix_calloc(d_old,d_old);
	gsl_vector * XtWY = gsl_vector_calloc(nc);
	gsl_matrix * RedX = gsl_matrix_calloc(nc,d_old) ;
	gsl_matrix * nSigmaU = gsl_matrix_calloc(d_old, d_old) ;
	gsl_permutation * P = gsl_permutation_calloc(d_old);
	gsl_vector * b_old = gsl_vector_calloc(d_old);
	gsl_vector * b_new = gsl_vector_calloc(d_old);
	int i=0,j=0;
	double denom=0.0;
	
	for (j=0;j<d_old;j++) b_old->data[j] = *(beta_old+j);
	
	for (i=0;i<nc;i++) 
	{
		W->data[i* W->tda + i] = wtc->data[i] ;
		denom += wtc->data[i] ;
	}
	
	GetReducedX(RedX, non0ind_old, d_old, Xc) ; // Reduced X

	//EvalnSigmaU(nSigmaU, lambda, 3.7, b_old, METH, (double) nc) ; 
	EvalnSigmaU(nSigmaU, lambda, 3.7, b_old, METH, denom) ; 
        
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,Xc,W,0.0,XtW);
	gsl_blas_dgemv(CblasNoTrans, 1.0, XtW, Yc, 0.0, XtWY) ; // X'Y
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, XtW, RedX, 0.0, XtWX) ; // X'X
	gsl_matrix_add(nSigmaU, XtWX) ; // (X'X + nSigmaU)
        
	gsl_linalg_LU_decomp(nSigmaU,P,&signum);
	gsl_linalg_LU_solve(nSigmaU,P,XtWY,b_new);
		
	for (j=0;j<d_old;j++) *(beta_new+j) = b_new->data[j] ;
	
	gsl_vector_free(b_old);
	gsl_vector_free(b_new);
	gsl_matrix_free(nSigmaU);
	gsl_matrix_free(RedX);
	gsl_matrix_free(XtWX);
	gsl_vector_free(XtWY);
	gsl_permutation_free(P);
	gsl_matrix_free(W);
	gsl_matrix_free(XtW);
	
	return 0 ;
}


