/*
 *  R1_IthContr_to_Proj.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/28/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 * cm1 = E[(X_1,X_2)|Vobs] = 2-by-1 vector
 * cm2 = E[(X_1,X_2)*(X_1,X_2)'|Vobs] = 2-by-2 matrix
 * Vobs = (Y,X_3,X_4,...,X_d) = (d-1)-by-1 vector
 * beta = must be d-by-1 !!!
 * 
 * score = E(X'*X*beta - X'*Y | Vobs) = d-by-1 vector
 */

#include "coarse.h"

int R1_IthContr_to_Proj(gsl_vector * score, gsl_vector * beta, gsl_vector * cm1, gsl_matrix * cm2, 
	gsl_vector * Vobs)
{
	size_t d = beta->size ;
	double Y = Vobs->data[0] ;
	gsl_vector * score_M = gsl_vector_calloc(2);
	gsl_vector * beta_M = gsl_vector_calloc(2);
	gsl_vector * beta_obs = gsl_vector_calloc(d-2);
	gsl_vector * X_obs = gsl_vector_calloc(d-2);
	gsl_vector * score_obs = gsl_vector_calloc(d-2);
	double pred_obs = 0.0; // X_obs * beta_obs
	double pred_M = 0.0 ; // cm1 * beta_M
	int j ;
	
	// setup
	beta_M->data[0] = beta->data[0] ;
	beta_M->data[1] = beta->data[1] ;
	
	for (j=0;j<(d-2);j++) 
	{
		beta_obs->data[j] = beta->data[j+2] ;
		X_obs->data[j] = Vobs->data[j+1] ;
	}
	
	pred_obs = 0.0 ;
	for (j=0;j<(d-2);j++) pred_obs += X_obs->data[j] * beta_obs->data[j] ;
	pred_M = 0.0 ;
	for (j=0;j<2;j++) pred_M += cm1->data[j] * beta_M->data[j] ;
	
	// calculations
	gsl_blas_dgemv(CblasNoTrans,1.0,cm2,beta_M,0.0,score_M);
	for (j=0;j<2;j++)
	{
		score_M->data[j] = score_M->data[j] + cm1->data[j] * (pred_obs - Y) ;
	}
	
	for (j=0;j<(d-2);j++)
	{
		score_obs->data[j] = X_obs->data[j] * (pred_M + pred_obs - Y) ;
	}
	
	score->data[0] = score_M->data[0] ;
	score->data[1] = score_M->data[1] ;
	for (j=0;j<(d-2);j++) score->data[2+j] = score_obs->data[j] ;
	
	gsl_vector_free(beta_M);
	gsl_vector_free(beta_obs);
	gsl_vector_free(score_obs);
	gsl_vector_free(score_M);
	gsl_vector_free(X_obs);
	return 0 ;
}

