/*
 *  ReadWages.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 4/11/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int ReadWages(MissBin * MB)
{
	int local_n = 534 ;
	int local_d = 17 ;
    gsl_matrix * AllDat = gsl_matrix_calloc(local_n,11) ;
	gsl_matrix * AllX = gsl_matrix_calloc(local_n,local_d);
	double logwage[534] ;
	double sum_wage = 0.0 ;
	double mean_wage = 0.0 ;
	int na_cntr = 0 ;
	int flag = 0 ;
	int i,j;
    FILE * fin = fopen("/Users/brentjohnson/Documents/Datasets/wages/wages_data.txt","r") ;
    if (fin == NULL)
    {
        printf("Cannot open data wages data file\n") ;
        exit(8) ;
    }
    
    gsl_matrix_fscanf(fin, AllDat) ;
	
	for (i=0;i<local_n;i++)
	{
		double educ = AllDat->data[i*AllDat->tda + 0] ;
		double south = AllDat->data[i*AllDat->tda + 1] ;
		double sex = AllDat->data[i*AllDat->tda + 2] ;
		double exper = AllDat->data[i*AllDat->tda + 3] ;
		double unio = AllDat->data[i*AllDat->tda + 4] ;
		double wage = AllDat->data[i*AllDat->tda + 5] ;
		double age = AllDat->data[i*AllDat->tda + 6] ;
		double race = AllDat->data[i*AllDat->tda + 7] ;
		double job = AllDat->data[i*AllDat->tda + 8] ;
		double sector = AllDat->data[i*AllDat->tda + 9] ;
		double marr = AllDat->data[i*AllDat->tda + 10] ;
		double sect1 = 0.0, sect2 = 0.0 ;
		double job1 = 0.0, job2 = 0.0 ;
		double white = 0.0 ;
		double educsouth, sexsouth, sexexper, job1south, job2south ;
		double sexsect1, job1union, job2white, marrjob1 ;
		
		if (isnan(educ)|isnan(south)|isnan(sex)) 
		{
			flag = 1 ;
			na_cntr++;
		}
		
		logwage[i] = 0.0 ;
		if (wage > 0.0) 
		{
			double lwage = log(wage);
			logwage[i] = lwage ;
			sum_wage += lwage ;
		}
		
		// new indicators
		if (baj_abs(sector-1.0)<1.E-4) sect1 = 1.0 ;
		else if (baj_abs(sector-2.0)<1.E-4) sect2 = 1.0 ;
		
		if ((baj_abs(job-1.0)<1.E-4)|(baj_abs(job-5.0)<1.E-4)) job1 = 1.0 ;
		else if ((baj_abs(job-4.0)<1.E-4)) job2 = 1.0 ;
		
		if (baj_abs(race-3.0)<1.E-4) white = 1.0 ;
		
		// new interactions
		educsouth = educ * south ;
		sexsouth = sex * south ;
		sexexper = sex * exper ;
		job1south  = job1 * south ;
		job2south = job2 * south ;
		sexsect1 = sex * sect1 ;
		job1union = job1 * unio ;
		job2white = job2 * white ;
		marrjob1 = marr * job1 ;
		
		AllX->data[i*AllX->tda + 0] = educ ;
		AllX->data[i*AllX->tda + 1] = sex ;
		AllX->data[i*AllX->tda + 2] = south ;
		AllX->data[i*AllX->tda + 3] = exper ;
		AllX->data[i*AllX->tda + 4] = unio ;
		AllX->data[i*AllX->tda + 5] = white ;
		AllX->data[i*AllX->tda + 6] = sect1 ;
		AllX->data[i*AllX->tda + 7] = sect2 ;
		AllX->data[i*AllX->tda + 8] = marr ;
		AllX->data[i*AllX->tda + 9] = job1 ;
		AllX->data[i*AllX->tda + 10] = job2 ;
		//AllX->data[i*AllX->tda + 11] = educsouth ;
		//AllX->data[i*AllX->tda + 12] = sexsouth ;
		//AllX->data[i*AllX->tda + 13] = sexexper ;
		AllX->data[i*AllX->tda + 11] = job1south ;
		AllX->data[i*AllX->tda + 12] = job2south ;
		AllX->data[i*AllX->tda + 13] = sexsect1 ;
		AllX->data[i*AllX->tda + 14] = job1union ;
		AllX->data[i*AllX->tda + 15] = job2white ;
		AllX->data[i*AllX->tda + 16] = marrjob1 ;
		
	}
	
	//printf("na_cntr = %d\n", na_cntr);
	
	//printf("sum_wage = %6.3f \n", sum_wage);
	mean_wage = sum_wage/((double) local_n) ;
	//printf("mean_wage = %6.3f \n", mean_wage);
	
	
	for (i=0;i<local_n;i++) 
	{
		MB->Y->data[i] = logwage[i] - mean_wage ;
		/*
		if (i<10)
		{
			printf("log(wage)[%d] = %6.3f\n", i, *(logwage+i));
		}
		*/
	}
	
	
	NormalizeX(AllX);	
	gsl_matrix_memcpy(MB->X,AllX);

	/*
	for (i=0;i<10;i++)
	{
		printf("Y[%d] = %6.3f, ",i, MB->Y->data[i]); 
		for (j=0;j<MB->X->size2;j++) printf("%4.1f ",gsl_matrix_get(MB->X,i,j));
		printf("\n");
	}
	*/
	
	free(AllDat);
	free(AllX);
	fclose(fin);
	
	return 0 ;
}

