/*
 *  ScoreHessianIPW.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/19/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int ScoreHessianIPW(gsl_vector * score, gsl_matrix * hess, gsl_vector * beta, 
	MissBin * MB, gsl_vector * Proj)
{
	int i,j,k ;
	size_t n = MB->G->size1 ;
	size_t d = MB->G->size2 ;	
	gsl_vector * score_local = gsl_vector_calloc(d);
	gsl_matrix * hess_local = gsl_matrix_calloc(d,d);
	
	for (i=0;i<n;i++)
	{
		if (MB->R->data[i] > 2.5)
		{
			double lp = 0.0 ;
			gsl_vector_view tmp_x = gsl_matrix_row(MB->G,i);
			gsl_blas_ddot(&tmp_x.vector, beta, &lp);
			
			double res =  MB->Y->data[i] - lp ;
			double lam1 = MB->lamhat->data[i*MB->lamhat->tda + 0] ;
			double lam2 = MB->lamhat->data[i*MB->lamhat->tda + 1] ;
			double pihat = (1.0-lam1)*(1.0-lam2) ;
			for (j=0;j<d;j++) 
			{
				double xj = MB->G->data[i*MB->G->tda + j] ;
				
				score_local->data[j] += -1.0 * res*xj/pihat ; // X'*X*beta - X'*Y = X'*(mu-Y)
				
				for (k=0;k<d;k++) 
				{
					hess_local->data[j*hess_local->tda + k] += 
						MB->G->data[i*MB->G->tda + k] * xj / pihat ;
				}
				
			}
		}
	}
	
	
	gsl_vector_sub(score_local,Proj);
	
	gsl_vector_memcpy(score,score_local);
	gsl_matrix_memcpy(hess,hess_local);
	
	gsl_vector_free(score_local);
	gsl_matrix_free(hess_local);
	
	return 0 ;
}

