/*
 *  WLS_IPW.c
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/23/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "coarse.h"

int WLS_IPW(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc)
{
	size_t nc = Xc->size1 ;
	size_t d = Xc->size2 ;
	gsl_matrix * W = gsl_matrix_calloc(nc,nc);
	gsl_matrix * XtW = gsl_matrix_calloc(d,nc);
	gsl_matrix * XtWX = gsl_matrix_calloc(d,d);
	gsl_vector * XtWY = gsl_vector_calloc(d);
	gsl_permutation * P = gsl_permutation_calloc(d);
	gsl_vector * yhat = gsl_vector_calloc(nc);
	gsl_matrix * diag = gsl_matrix_calloc(d,d);
	gsl_matrix * V = gsl_matrix_calloc(d,d);
	double sse=0.0,mse=0.0,denom=0.0;
	int signum ;
	int i,j ;
	
	for (j=0;j<d;j++) diag->data[j * diag->tda + j] = 1.0 ;
	for (i=0;i<nc;i++) W->data[i* W->tda + i] = wtc->data[i] ;
	
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1.0,Xc,W,0.0,XtW);
	gsl_blas_dgemv(CblasNoTrans,1.0,XtW,Yc,0.0,XtWY);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,XtW,Xc,0.0,XtWX);
	
	gsl_linalg_LU_decomp(XtWX,P,&signum);
	gsl_linalg_LU_solve(XtWX, P, XtWY, bhat);
    
    gsl_blas_dgemv(CblasNoTrans, 1.0, Xc, bhat, 0.0, yhat);
	
	sse = 0.0 ;
	denom = 0.0 ;
    for (i=0;i<nc;i++) 
	{
		sse += pow((Yc->data[i] - yhat->data[i]), 2.0) * wtc->data[i];
		denom += wtc->data[i] ;
	}
    mse = sse/denom;
    //mse = sse/175.;

    for (j=0 ; j<d ; j++)
    {
        gsl_vector_const_view d_j = gsl_matrix_const_column(diag,j) ;
        gsl_vector_view res_j = gsl_matrix_column(V,j) ;
        gsl_linalg_LU_solve(XtWX, P, &d_j.vector, &res_j.vector) ;
    }
	
    for (j=0;j<d;j++) sehat->data[j] = sqrt(mse * V->data[j*V->tda + j]) ;
    

    gsl_matrix_free(W);
	gsl_matrix_free(XtW);
	gsl_matrix_free(XtWX) ;
    gsl_vector_free(XtWY);
    gsl_matrix_free(V);
    gsl_matrix_free(diag);
    gsl_vector_free(yhat);
    gsl_permutation_free(P);
	return 0 ;
}

