/*
 *  coarse.h
 *  CoarsenedData
 *
 *  Created by Brent Johnson on 3/19/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_roots.h>
#include <gsl/gsl_multiroots.h>
#include <gsl/gsl_permute.h>
#include <gsl/gsl_permute_vector.h>
#include <gsl/gsl_min.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_multifit.h>


typedef struct BIN_FOR_COARSE
{
	int nr_eq1 ;			// number with R=1
	int nr_eq2 ;			// number with R=2
	int nr_eq99 ;			// number of complete cases
    gsl_vector * Y ;		// n-by-1 vector
    gsl_matrix * X ;		// n-by-d matrix
    gsl_vector * R ;		// n-by-1 vector, 1 = miss {x1,x2}, 2 = miss {x1}, 99 complete data 
	gsl_matrix * G ;		// put zeros if missing, X otherwise
	gsl_vector * bhatI ;	// IPW estimator initial value
	gsl_vector * L2 ;		// L2 projection
	gsl_vector * psi1 ;		// coarsening parameters
	gsl_vector * psi2 ;
	gsl_vector * theta_mu ;		// (d+1) x 1 vector, full model parameters, MVN mean
	gsl_matrix * theta_sigma ;	// (d+1) x (d+1) full model parameters, MVN covar
	//double m1X1 ;
	//double m2X1 ;
	//gsl_vector * m1X12 ;
	//gsl_matrix * m2X12 ;
	gsl_matrix * lamhat ; // n x 2 matrix, missingness probabilities
} MissBin ;

int AunderB(const gsl_matrix * A, const gsl_matrix * B, gsl_matrix * result);

double baj_abs(double v);
int baj_print_vector(gsl_vector * v);
int baj_print_matrix(gsl_matrix * v);

int CalculateCellProbs(MissBin * MB);
int CovarianceX(gsl_matrix * V);

int ElasticNet_Eff(MissBin * MB, double lambda1, double lambda2, 
	gsl_vector * bhat, gsl_vector * sehat, int se_flag);

int EstimateBetaEFF(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Yc, 
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB);
int EstimateConditionalMomentsR2(double * cm1, gsl_vector * Vobs, 
	double mu1, gsl_vector * Sigma12, gsl_matrix * Sigma22, gsl_vector * mu2);
int EstimatePsi(MissBin * MB);
int EstimateTheta(MissBin * MB);
double EvalPbeta(double lambda, double a1, double abj, int METH);
int EstimateL2Projection(gsl_vector * Proj, gsl_vector * beta, MissBin * MB);
int EstimateProjReduced(gsl_vector * proj_red, gsl_vector * beta_old, int d_old, int * non0ind_old, 
	MissBin * MB);

int EstimateVhatEffScoreVar(gsl_matrix * I1, gsl_vector * beta, MissBin * MB);
int EstimateVhatEffGradient(gsl_matrix * I0, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, 
	MissBin * MB);
int EstimateVarBhatEff(gsl_vector * sehat, gsl_vector * bhat, gsl_vector * Yc, 
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB);

int EvalnSigmaU(gsl_matrix * nSigmaU, double lambda, double a1, gsl_vector * beta, int METH, double n);
int EvalnSigmaU2(gsl_matrix * nSigmaU, double lambda, double a1, gsl_vector * beta, int METH, double n, 
	gsl_vector * b0);

int ExtractVobsForR1(gsl_vector * Vobs, int i, gsl_vector * Y, gsl_matrix * G);
int ExtractVobsForR2(gsl_vector * Vobs, int i, gsl_vector * Y, gsl_matrix * G);

int Find_OLS_Eff_Thresholds(double * th_new, int d_new, int * non0ind_new, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB) ;

int FindPenStdErrorsOLSFull(gsl_vector * se, gsl_vector * b, int * non0ind_new, int d_new, 
	gsl_vector * Y, gsl_matrix * X, double lambda, int METH);
int FindPenStdErrorsOLSEff(gsl_vector * se, gsl_vector * b_all, int * non0ind_new, int d_new, 
	double lambda, int METH, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, 
	gsl_vector * b0);
	
int GCV_ElasticNet_Eff(double * lam1_ptr, double * lam2_ptr, MissBin * MB);
double GCV_OLS_Full(gsl_vector * b0, gsl_vector * se0, gsl_vector * Y, gsl_matrix * X, int METH, 
	int NSTEP);
double GCV_OLS_Eff(gsl_vector * b0, gsl_vector * se0, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, int METH, 
	int NSTEP);
double GCV_OLS_Eff_Wages(gsl_vector * b0, gsl_vector * se0, 
	gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, int METH, 
	int NSTEP, double L_lo, double L_hi);

int GetScoreEffFullMatrix(gsl_matrix * D, gsl_vector * beta, MissBin * MB);

int GetCompleteYX(gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB);
int GetLamRange(double * lambda, int len_lambda, gsl_vector * b0, gsl_vector * Y, gsl_matrix * X, 
	int METH);
int GetReducedX(gsl_matrix * RedX, int * keep_index, int n_keep, gsl_matrix * X);

double GetTrProj(gsl_vector * b, gsl_matrix * X, double lambda, int METH);
double GetTrProjIPW(gsl_vector * b, gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, double lambda1, 
	double lambda2, int METH, gsl_vector * b0);

int InitAllocMissBin(int n, int d, MissBin * MB);

int LogisticRegression(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Y, gsl_matrix * X);
int LSFit(gsl_vector * b, gsl_vector * se, gsl_vector * y, gsl_matrix * X);

int MatrixSquareRoot(gsl_matrix * Root, gsl_matrix * M);
double MaxAbsDiff(gsl_vector * v1, gsl_vector * v2);

int NewtonRaphsonFull(gsl_vector * bhat, gsl_vector * sehat, MissBin * MB, gsl_vector * beta_init);
int NormalizeX(gsl_matrix * X);

int OLSEFFBlock(double * bhat, double * sehat, MissBin * MB, int * non0ind, int n_nonzero);
int OLSEFFBlockWages(double * bhat, double * sehat, MissBin * MB, int * non0ind, int n_nonzero);
int OLSEFFBlockWages11(double * bhat, double * sehat, MissBin * MB, int * non0ind, int n_nonzero);

int OLS_Eff_FITTER(gsl_vector * b0, gsl_vector * se0, gsl_vector * Yc, 
	gsl_matrix * Xc, gsl_vector * wtc, MissBin * MB, 
    double lambda, int METH, int NSTEP, gsl_vector * bret, gsl_vector * seret, int se_flag);
int OLS_Eff_Oracle(gsl_vector * bOR, gsl_vector * seOR, int * ind_keep, int len_keep, 
	MissBin * MB);

int OLSFullBlock(double * bhat, double * sehat, gsl_vector * Y, gsl_matrix * X, 
	int * non0ind, int n_nonzero);
int OLSFullBlockWages(double * bhat, double * sehat, gsl_vector * Y, gsl_matrix * X, 
	int * non0ind, int n_nonzero);
int OLSFullBlockWages11(double * bhat, double * sehat, gsl_vector * Y, gsl_matrix * X, 
	int * non0ind, int n_nonzero);
int OLS_Full_FITTER(gsl_vector * b0, gsl_vector * se0, gsl_vector * Y, gsl_matrix * X, 
    double lambda, int METH, int NSTEP, gsl_vector * bret, gsl_vector * seret, int se_flag);
int OLS_Full_Oracle(gsl_vector * bOR, gsl_vector * seOR, int * ind_keep, int len_keep, 
	gsl_vector * Y, gsl_matrix * X);

int OLSIPWBlock(double * bhat, double * sehat, MissBin * MB, int * non0ind, int n_nonzero);
int OLS_IPW_FITTER(gsl_vector * b0, gsl_vector * se0, gsl_vector * Yc, gsl_matrix * Xc, 
	gsl_vector * wtc, MissBin * MB, 
    double lambda, int METH, int NSTEP, gsl_vector * bret, gsl_vector * seret, int se_flag);

int OLSNaiveBlock(double * bhat, double * sehat, MissBin * MB, int * non0ind, int n_nonzero);

int PredictVmissLinearly_for_R1(gsl_matrix * alpha, gsl_matrix * tau_sq, MissBin * MB);
int PredictVmissLinearly_for_R2(gsl_vector * alpha, double * tau_sq, MissBin * MB);

int R1_PredictiveFunction(gsl_vector * cm1, gsl_matrix * cm2, gsl_matrix * alpha_R1, 
	gsl_matrix * tau_sq_R1, gsl_vector * Vobs);
int R1_IthContr_to_Proj(gsl_vector * score, gsl_vector * beta, gsl_vector * cm1, gsl_matrix * cm2, 
	gsl_vector * Vobs);
	
int R2_PredictiveFunction(double * cm1, double * cm2, gsl_vector * alpha_R2, double tau_sq_R2, 
	gsl_vector * Vobs);
int R2_IthContr_to_Proj(gsl_vector * score, gsl_vector * beta, double cm1, double cm2, 
	gsl_vector * Vobs);

int R2_Mu_Rest(gsl_vector * mu, MissBin * MB);
int R2_Sigma_X1_Rest(gsl_vector * S, MissBin * MB);
int R2_Sigma_Rest(gsl_matrix * S, MissBin * MB);

int ReadCmdLine(int argc, char ** argv);
int ReadSimSeeds(int * simseeds);
int ReadWages(MissBin * MB);
double ResidualSSWeighted(gsl_vector * Yc,gsl_matrix * Xc,gsl_vector * wtc,
	gsl_vector * beta, MissBin * MB);

int SandwichEstimate(gsl_matrix * V, gsl_matrix * A, gsl_matrix * B);

int Shrink_OLS_Full_Bhat(gsl_vector * b_new, gsl_vector * Y, gsl_matrix * X, 
	gsl_vector * b_old, double lambda, int METH, gsl_vector * b0);
int Shrink_OLS_Eff_Bhat(gsl_vector * beta_new, MissBin * MB, 
	gsl_vector * beta_old, double lambda, int METH, gsl_vector * b0);

int SimStudy(const gsl_rng * r, int baj_index);
int SimulateYX(const gsl_rng * r, gsl_vector * Y, gsl_matrix * X, 
	int * non0ind, int * n_nonzero);
int SimulateRG(const gsl_rng * r, MissBin * MB);
int SimulateRG_Wages(const gsl_rng * r, MissBin * MB);
int SimulateRG_Wages11(const gsl_rng * r, MissBin * MB);

int ScoreHessianIPW(gsl_vector * score, gsl_matrix * hess, gsl_vector * beta, 
	MissBin * MB, gsl_vector * Proj);
int ScoreHessianFull(gsl_vector * score, gsl_matrix * hess, gsl_vector * beta, MissBin * MB);
int ScoreHessianEffReduced(gsl_vector * score, gsl_matrix * hess, 
	gsl_vector * beta_old, MissBin * MB);

int UpdateLambda(MissBin * MB);

int WagesAnalysis(const gsl_rng * r);
int WagesAnalysis11(const gsl_rng * r);
int WagesSimulation(const gsl_rng * r);
int WLS_IPW(gsl_vector * bhat, gsl_vector * sehat, gsl_vector * Yc, gsl_matrix * Xc, gsl_vector * wtc);
